#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
SQL for StarDict
Created at: 2010-01-06
This version of the tool is just an idea.
The software is made public in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.
This version is conceived specifically for dexonline sqldump.
Not for sqldumps in general!
The dexonline database is under the
Copyright (C) 2004-2010 DEX online
http://dexonline.ro/download/dex-database.sql.gz
You must mention the dexonline copyright.
See also the conditions of the DEX online project.
Please always work with copies of the files!
The software has few safeguards against improper use and
there might be unexpected effects of the software.
Author: Mihail-Radu Solcan
Visit http://www.ub-filosofie.ro/~solcan/wt/wtc/gnu/index.html
"""
__copyright__ = "GPL"
__revision__ = "0.1"
__date__ = "2010-01-27"

import sys
import re
import csv
import os
import time

# Verifică sufixul numelui fișierului
def este_sql(rand_ltx):
    """ Funcţie booleană pentru sufixul fișierului prelucrat """
    return rand_ltx.endswith('.sql')

# Funcţiile următoare servesc la identificarea unor comenzi
# din documente SQL. Presupunem că ele apar pe prima poziţie din
# rând. 
def este_definition(rand_ltx):
    """ Funcţie booleană pentru definiții lexicale """
    return rand_ltx.startswith('INSERT INTO `Definition` VALUES')

# În sursa StarDict rândurile albe sunt, de fapt, comenzi.
# Rândul alb cere construirea unei noi intrări în dicționar.
def este_rand_alb(rand_ltx):
    """ Funcţie booleană pentru a stabili dacă rândul este alb """
    if rand_ltx.startswith('\n'):
        return 1
    else:
        return 0

def subst_final_sql_insert(abc):
    """ Substituţie pentru finalul de la INSERT INTO """
    verbal = re.compile('\);$')
    abc = verbal.sub('\n', abc)
    return abc

def vb_nr_elimina_spatii(abc):
    """ Substituţie pentru a nu crea confuzii cu grupuri de sensuri """
    verbal = re.compile(' vb\\.[ ]*((IX|IV|V?I{0,3})\\.) ')
    abc = verbal.sub(' vb.\\1 ', abc)
    return abc

def bold_ro_sens(abc):
    """ Substituţie pentru a pune bold la numere romane pana la IX """
    verbal = re.compile(' ((IX|IV|V?I{0,3})\\.) ')
    abc = verbal.sub(' <b>\\1</b> ', abc)
    return abc

def linie_noua_ro_sens(abc):
    """ Substituţie pentru a trece grupuri de sensuri pe un alt rand """
    verbal = re.compile('(<b>(IX|IV|V?I{0,3})\\.</b>)')
    abc = verbal.sub('<br>\\1', abc)
    return abc

def bold_sens(abc):
    """ Substituţie pentru a pune bold la numere arabe """
    verbal = re.compile(' ([0-9]{1,2}\\.) ')
    abc = verbal.sub(' <b>\\1</b> ', abc)
    return abc

def linie_noua_sens(abc):
    """ Substituţie pentru a trece sensurile pe un alt rand """
    verbal = re.compile('(<b>[0-9]{1,2}\\.</b>)')
    abc = verbal.sub('<br>\\1', abc)
    return abc

def subst_href(abc):
    """ Pune trimitere internă """
    verbal = re.compile('<a class=\\\\"ref\\\\" href=\\\\"/search\\.php\\?cuv=(.*?)\\\\">(.*?)</a>')
    abc = verbal.sub('<a href="bword://\\1">\\2</a>', abc)
    return abc

# StarDict foloseşte UTF-8, care este preferabil entităților html!
# Înțelege html cu opțiunea sametypesequence=h în versiunea 3.0.0
# Definim o functie pentru subtituţii pentru compatibilizare cu StarDict
def pune_semne_stardict(semne):
    """ Introduce semne compatibile cu StarDict """
    semne = semne.replace('/&lt;bulg', '← bulg')
    semne = semne.replace('/&lt;engl', '← engl')
    semne = semne.replace('/&lt;fr', '← fr')
    semne = semne.replace('/&lt; fr', '← fr')
    semne = semne.replace('(&lt; fr', '(← fr')
    semne = semne.replace('/&lt;germ', '← germ')
    semne = semne.replace('/&lt;it', '← it')
    semne = semne.replace('/&lt;lat', '← lat')
    semne = semne.replace('/&lt; lat', '← lat')
    semne = semne.replace('/&lt;l<i>at', '← lat')
    semne = semne.replace('/&lt;ngr', '← ngr')
    semne = semne.replace('/&lt;pol', '← pol')
    semne = semne.replace('/&lt;rus', '← rus')
    semne = semne.replace('/&lt;sb', '← sb')
    semne = semne.replace('/&lt;sl', '← sl')
    semne = semne.replace('/&lt;turc', '← turc')
    semne = semne.replace('/&lt;țig', '← țig')
    semne = semne.replace('/&lt;ucr', '← ucr')
    semne = semne.replace('/&lt;ung', '← ung')
    semne = semne.replace('conjcț.', 'conj.')
    semne = semne.replace('/</', '</')
    semne = vb_nr_elimina_spatii(semne)
    semne = bold_ro_sens(semne)
    semne = linie_noua_ro_sens(semne)
    semne = bold_sens(semne)
    semne = linie_noua_sens(semne)
    semne = subst_href(semne)
    return semne

# StarDict are motor html
# Definim o functie pentru retușarea html
def corect_html(semne):
    """ Introduce marcaj html compatibil cu StarDict """
    semne = semne.replace('&nbsp;', ' ')
    semne = semne.replace('<b></i>', '</i><b>')
    semne = semne.replace('</b>[', '[</b>')
    return semne

# StarDict are motor html
# Dar sunt probleme cu entititățile xml; le convertim la UTF
def ent_xml(abc):
    """ Convertim entitățile xml la UTF-8 """
    """ Felul în care manevrează însă python cu UTF-8 este tricky"""
    """ De folosit substituții simple sau cele manuale în caz de erori """
    xmlent = re.compile('(&#x[0-9A-F]{4};)')    # atn.la greedy
    cod_hex = re.compile('&#x([0-9A-F]{4});')
    lista = xmlent.split(abc)
    abc_nou = ''
    for index in range(len(lista)):
        if re.search('&#x', lista[index]):       # este și o precauție aici
            caracter = u''
            lista[index] = cod_hex.sub('\\1', lista[index])
            lista[index] = unichr(int(lista[index],16))
            caracter += lista[index]
            caracter = caracter.encode('utf-8')  # sucelile Python-ului?
            abc_nou += caracter                  # StarDict folosește UTF-8 !!!
        else:
            abc_nou += lista[index]
#    Am putea proceda desigur mult mai simplu când nu este
#    vorba decât despre trei-patru entități,
#    abc = abc.replace('&#x2013;', '–')
#    abc = abc.replace('&#x2662;', '♢')
#    abc = abc.replace('&#x2666;', '♦')
#    return abc
#    S-ar putea însă să avem multe substituții și atunci
#    este mai comod să recurgem la expresii regulate
    return abc_nou

# Identificarea dicționarului din care provine definiția
def src_dic(id):
    abrev = ''
    if id == '1':
        abrev = '[DEX98]'
    elif id == '2':
        abrev = '[DEX96]'
    elif id == '3':
        abrev = '[DEX84]'
    elif id == '4':
        abrev = '[DEX75]'
    elif id == '5':
        abrev = '[DEX-S88]'
    elif id == '6':
        abrev = '[Sinonime]'
    elif id == '7':
        abrev = '[Antonime]'
    elif id == '8':
        abrev = '[DOR]'
    elif id == '9':
        abrev = '[NODEX]'
    elif id == '10':
        abrev = '[DAR]'
    elif id == '11':
        abrev = '[DGE]'
    elif id == '12':
        abrev = '[DER]'
    elif id == '13':
        abrev = '[DLRA]'
    elif id == '14':
        abrev = '[DLRC]'
    elif id == '15':
        abrev = '[DLRM]'
    elif id == '16':
        abrev = '[DMLR]'
    elif id == '17':
        abrev = '[DN]'
    elif id == '18':
        abrev = '[DOOM]'
    elif id == '19':
        abrev = '[DOOM2]'
    elif id == '20':
        abrev = '[MDA]'
    elif id == '21':
        abrev = '[MDN]'
    elif id == '22':
        abrev = '[Neoficial]'
    elif id == '23':
        abrev = '[DLRLV]'
    elif id == '24':
        abrev = '[DLRLC]'
    elif id == '25':
        abrev = '[DE]'
    else:
        abrev = '[alte surse]'

    return abrev

# Extragerea definițiilor lexicale într-un fișier csv
def creeaza_csv_def(text_in):
    """ Se construiește un fișier csv  """
    """ campuri_csv ='Id,UserId,SourceId,Lexicon,Displayed,InternalRep,HtmlRep,Status,CreateDate,ModDate,ModUserId' """
    pretext = [] # iniţializarea variabilei pentru rezultatul preprocesării
    for linie in text_in:
        if not este_rand_alb(linie):# omite eventuale rânduri albe
            linie = linie.strip()   # elimină spaţiile albe iniţiale/finale
        if este_definition(linie):
            linie = linie.replace('INSERT INTO `Definition` VALUES (', '')
            linie = linie.replace('),(', '\n')   # câmpurile cu valori
            linie = subst_final_sql_insert(linie)# rămân separate doar de 
            pretext += [linie]                   # virgule
    return pretext                               # textul între '\x27'

def este_art_ok(item):
    """ Funcţie booleană pentru selectarea articolelor """
    nr_campuri = len(item)
    if nr_campuri != 11:                       # itemi fără structură standard
        print 'itemul ' + item[0] + ' are ' + str(nr_campuri) + ' câmpuri'
        art_nonstd = ''
        for i in range(nr_campuri):
            if i != 0:
                art_nonstd += '_'              # separator non-standard
            art_nonstd += item[i]
        art_nonstd += '\n'
        fisier_nonstd.write(art_nonstd)
    if nr_campuri == 11 and item[7] == '0':    # dacă numărul de câmpuri este
#        print 'itemul ' + item[0]             # standard și articolul este
        return 1                               # acceptabil în dicționar
    else:
        return 0

def extrage_forme(def_id, def_ttl):
    """ extrage forme gramaticale folosind harta corelațiilor"""
    """ dintre lexeme și definiții, precum și """
    """ listele de cuvinte """
    forme = ''
    lex_id = ''
    corelatii = os.popen('grep \'^' + def_id + ',\' ' + nume_fisier_lex_csv + ' | sed \'s/^.*,\([0-9]*\)/\\1/g\'','r')
    sys.stdout.flush()
    lex_id = corelatii.readlines()
    if len(lex_id) == 0:
        print def_id + ' fără lexem'
#    if len(lex_id) > 1: # cu mai mult de un lexem
#        for i in range(len(lex_id)):
#            polilex.write(def_id + ' are lexemul ' + lex_id[i].strip() + '\n')
    for i in range(len(lex_id)):
        lex_id[i] = lex_id[i].replace('\n', '')
        print def_id + ' cu lexemul ' + lex_id[i]
        if lex_id[i] != '':
            select = os.popen('grep -w \'^' + lex_id[i] + ',\' ' + nume_fisier_wl_csv + ' | sed \"s/^.*,\'\(.*\)\'.*/\\1/g\" | sort -u','r')
            sys.stdout.flush()
            lista_forme = select.readlines()
            len_lista_forme = len(lista_forme)
            if len_lista_forme >= 1:
                for j in range(len_lista_forme):
                    forma = lista_forme[j].strip()
                    if not re.search(forma, def_ttl):
                        forme += '|'
                        forme += forma
                        def_ttl += forme
    return forme

#Adaugă la toate sursele
def adauga_forme_peste_tot(id, ttl):
    """ forme pentru toate intrările din dicționar """
    ttl_plus_forme = ttl
    alte_forme = extrage_forme(id, ttl)   # pentru orice dicționar
    ttl_plus_forme += alte_forme          # durează trei sferturi de zi
    ttl_plus_forme += '\n'
    return ttl_plus_forme

#Adaugă doar la unele surse
def adauga_forme_selectiv(src, id, ttl):
    """ forme pentru unele intrări din dicționar """
    ttl_plus_forme = ttl
    if src[2] == '1':                     # alege dicționarul
        alte_forme = extrage_forme(id, ttl)
        ttl_plus_forme += alte_forme      # căruia i se atașează alte forme
        ttl_plus_forme += '\n'            # oricum va dura însă câteva ore
    else:
        ttl_plus_forme += '\n'
    return ttl_plus_forme

# Construiește un dicționar care include forme gramaticale
# Adăugarea altor forme gramaticale
# lungește considerabil procesul de prelucrare
def alege_titlu_art_cu_forme(srcart):
    """ construiește intrarea din dicționar """
    ttlart = ''
    id_art = ''
    id_art += srcart[0]                   # id-ul articolului de dicționar
    ttlart += srcart[3]                   # intrarea în sursa StarDic
    # Pentru forme peste tot se scoate comentariul la comanda
#    ttlart = adauga_forme_peste_tot(id_art, ttlart)
    # Pentru a avea forme doar selectiv se comentează intrarea anterioară
    # și se scoate comentariul la intrarea următoare
    ttlart = adauga_forme_selectiv(srcart, id_art, ttlart)
    return ttlart

# Construiește un dicționar simplu
# fără forme gramaticale
def alege_titlu_art_din(srcart):
    """ construiește intrarea din dicționar """
    ttlart = ''
    id_art = ''
    id_art += srcart[0]                   # id-ul articolului de dicționar
    ttlart += srcart[3]                   # intrarea în sursa StarDic
    ttlart += '\n'
    return ttlart

def alege_componente_art_din(srcart):
    """ extrage conținutul articolului """
    strart = ''
#    strart += srcart[0]+' '    # Id
#    strart += srcart[1]+' '    # UserId
#    strart += srcart[2]+' '    # SourceId
#    strart += srcart[3]+' '    # Lexicon
#    strart += srcart[4]+' '    # Displayed
#    strart += srcart[5]+' '    # InternalRep
    strart += srcart[6]+' '     # HtmlRep
#    strart += srcart[7]+' '    # Status
#    strart += srcart[8]+' '    # CreateDate
#    strart += srcart[9]+' '    # ModDate
#    strart += srcart[10]       # ModUserId
    dic = srcart[2]             # identifică sursa articolului
    dic = src_dic(dic)          # identifică abrevierea sursei
    strart += dic               # adaugă abreviere
    strart += '\n\n'
    return strart

# Extragerea articolelor de dicționar
def extrage_articole(lectura_def):
    """ Se pornește de la câmpurile fișierului csv """
    """ și se construiesc definiții conform sintaxei Babylon"""
    sursa = [] # lista elementelor corpului sursei de definiții
    for itemi in lectura_def:
        ttl = ''
        art = ''
        if este_art_ok(itemi):
            # Pentru forme comanda următoare este necomentată
#            ttl = alege_titlu_art_cu_forme(itemi)  # cu forme durează mult
            # Pentru dicționarul simplu
            ttl = alege_titlu_art_din(itemi)       # fără forme relativ puțin
            # Intrarea anterioară trebuie
            # comentată pentru a avea forme
            art = alege_componente_art_din(itemi)
            art = pune_semne_stardict(art)
            if re.search('&#x[0-9A-F]{4};', art):
                art = ent_xml(art)
            art = corect_html(art)
        sursa += [ttl]
        sursa += [art]
    return sursa

# S-a terminat definirea de funcţii.

# Definim constante pentru a putea scrie antetul sursei dicționarului
BLANK_LINE = '\n'
BOOK_NAME = '#bookname=dexonline.ro\n'
VERSION = '#version=3.0.0\n'
STRIP_METHOD = '#stripmethod=keep\n'
SAME_TYPE_SEQUENCE = '#sametypesequence=h\n'
DB_AUTHOR = '#author=\n'
SRC_AUTHOR_EMAIL = '#email=\n'
WEBSITE = '#website=\n'
SRC_DESCRIPTION = '#description=Acesta este o adaptare\
 a bazei de date DEX online sub licența GPL.\n'
DATA = '#date=' + time.strftime("%Y-%m-%d") + '\n'

# În urma apelului din linia de comandă la momentul
print time.ctime()
# începe procesul de prelucrare efectivă a fişierului SQL
if __name__ == "__main__":
    if len(sys.argv) != 2: 
        print 'Usage: %s file.sql' % (sys.argv[0]) 
        sys.exit(1)
    nume_fisier_in = sys.argv[1]        # preluarea numelui fişierului SQL
    if este_sql(nume_fisier_in):
        fisier = open(nume_fisier_in,'r')# deschiderea fişierului SQL
    else:
        print 'No sql dump!'
        sys.exit(1)
    text_sql = fisier.readlines()       # preluarea conţinutului fişierului
    fisier.close()                      # închiderea fişierului SQL

# Numele fișierelor de tip csv
    nume_fisier_lex_csv = nume_fisier_in.replace('.sql', '-lex.csv')
    nume_fisier_wl_csv = nume_fisier_in.replace('.sql', '-wl.csv')

# Este creat un fișier temporar de tip csv
# În fișierul csv sunt preluate definițiile lexicale din fișierul SQL
    text_csv_def = creeaza_csv_def(text_sql)
    nume_fisier_csv = nume_fisier_in.replace('.sql', '.csv')
    fisier_csv = open(nume_fisier_csv,'w')
    for i in range(len(text_csv_def)): 
        fisier_csv.write(text_csv_def[i])
    fisier_csv.close()

# Deschidem un fișier pentru inregistrari cu campuri csv non-standard
    nume_fisier_nonstd = 'tmp-nonstd.csv'
    fisier_nonstd = open(nume_fisier_nonstd, 'w')

# Deschidem un fișier pentru a detecta corelarea unei definiții
# cu mai multe lexeme. Numai când este efectiv nevoie!
#    polilex = open('polilex.txt', 'w')

# Deschidem fișierul cu definiții și le prelucrăm
    csv_def = open(nume_fisier_csv,'r')
    lector_csv = csv.reader(csv_def, delimiter=',', quotechar='\x27', doublequote=False, quoting=csv.QUOTE_ALL)
    articole = extrage_articole(lector_csv)

# În continuare este pregătit pentru scriere fişierul sursă al dicționarului
    nume_fisier_out = nume_fisier_in.replace('.sql', '.baby')
    fisier_out = open(nume_fisier_out,'w')

    # Scriem antetul sursei dicționarului StarDict
    fisier_out.write(BLANK_LINE)
    fisier_out.write(BOOK_NAME)
    fisier_out.write(VERSION)
    fisier_out.write(STRIP_METHOD)
    fisier_out.write(SAME_TYPE_SEQUENCE)
#    fisier_out.write(DB_AUTHOR)
#    fisier_out.write(SRC_AUTHOR_EMAIL)
#    fisier_out.write(WEBSITE)
    fisier_out.write(DATA)
    fisier_out.write(SRC_DESCRIPTION)
    fisier_out.write(BLANK_LINE)
    # Scriem textul sursei StarDict
    for i in range(len(articole)): 
        fisier_out.write(articole[i])
    
    # închidem fişierul cu sursa dicționarului
    fisier_out.close()
# Închidem fișierele temporare
    csv_def.close()
    fisier_nonstd.close()
#    polilex.close()
# Am încheiat prelucrarea fişierului SQL şi scrierea fişierului DIC
# Gata la momentul
print time.ctime()
# Final
