#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
Extract from DEX online SQL dump the correspondences between
lexical definitions and lexems.
Created at: 2010-01-11
This version is conceived specifically for dexonline sqldump.
Author: Mihail-Radu Solcan
Visit http://www.filosofie.unibuc.ro/~solcan/wt/wtc/gnu/index.html
"""
__copyright__ = "GPL"
__revision__ = "1"
__date__ = "2010-01-11"

import sys
import re
import os
import time

# Verifică sufixul numelui fișierului
def este_sql(rand_ltx):
    """ Funcţie booleană pentru sufixul fișierului prelucrat """
    return rand_ltx.endswith('.sql')

# Funcţiile următoare servesc la identificarea unor comenzi
# din documente SQL. Presupunem că ele apar pe prima poziţie din
# rând. 

def este_lex(rand_ltx):
    """ Funcţie booleană pentru lexeme """
    return rand_ltx.startswith('INSERT INTO `LexemDefinitionMap` VALUES')

# În sursa StarDict rândurile albe sunt, de fapt, comenzi.
# Rândul alb cere construirea unei noi intrări în dicționar.
def este_rand_alb(rand_ltx):
    """ Funcţie booleană pentru a stabili dacă rândul este alb """
    if rand_ltx.startswith('\n'):
        return 1
    else:
        return 0

def subst_final_sql_insert(abc):
    """ Substituţie pentru finalul de la INSERT INTO """
    verbal = re.compile('\);$')
    abc = verbal.sub('\n', abc)
    return abc

# Extragerea corelațiilor dintre lexeme și intrări într-un fișier csv
def creeaza_csv_lex(lista_in):
    """ Se construiește un fișier csv  """
    """ campuri_csv ='Id, LexemId, DefinitionId'  """
    lista_out = [] # iniţializarea variabilei pentru rezultatul preprocesării
    for linie in lista_in:
        if not este_rand_alb(linie):
            linie = linie.strip()   # elimină spaţiile albe iniţiale/finale
        if este_lex(linie):
            linie = linie.replace('INSERT INTO `LexemDefinitionMap` VALUES (', '')
            linie = linie.replace('),(', '\n')
            linie = subst_final_sql_insert(linie)
            lista_out += [linie]
    return lista_out

# S-a terminat definirea de funcţii.

# În urma apelului din linia de comandă la momentul
print  time.clock()
# începe procesul de prelucrare efectivă a fişierului SQL
if __name__ == "__main__":
    nume_fisier_in = sys.argv[1]        # preluarea numelui fişierului SQL
    if este_sql(nume_fisier_in):
        fisier = open(nume_fisier_in,'r')# deschiderea fişierului SQL
    else:
        print 'No sql dump!'
        exit()
    text_sql = fisier.readlines()       # preluarea conţinutului fişierului
    fisier.close()                      # închiderea fişierului SQL

# Este creat un fișier de tip csv
# În fișierul csv este preluată 
# corelarea lexemelor cu definițiile din fișierul SQL
    text_csv_lex = creeaza_csv_lex(text_sql)
    nume_fisier_lex_all = nume_fisier_in.replace('.sql', '-lex-all.csv')
    nume_fisier_lex = nume_fisier_in.replace('.sql', '-lex.csv')
    fisier_lex_all = open(nume_fisier_lex_all,'w')
    for i in range(len(text_csv_lex)): 
        fisier_lex_all.write(text_csv_lex[i])
    fisier_lex_all.close()
    # În fișierul mic, păstrăm numai ultimele două câmpuri (inversate)
    doua_campuri = os.system('cat ' + nume_fisier_lex_all + '| sed \'s/[0-9]*,\([0-9]*\),\([0-9]*\)$/\\2,\\1/g\' | sort -g > ' + nume_fisier_lex + '')
    sterge_lex_all = os.system('rm -vf ' + nume_fisier_lex_all + '')
    print 'stardex-tools use `%s`' % (nume_fisier_lex)

# La momentul
print time.clock()
# Este gata.
