#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
Create src in babylon format for StarDict
Created at: 2010-01-27
This version of the tool is just an idea.
This version is conceived specifically for dexonline sqldump.
Author: Mihail-Radu Solcan
Visit http://www.ub-filosofie.ro/~solcan/wt/wtc/gnu/index.html
"""
__copyright__ = "GPL"
__revision__ = "1"
__date__ = "2010-01-27"

import sys
import re
import time

# Verifică sufixul numelui fișierului
def este_baby(rand_ltx):
    """ Funcţie booleană pentru sufixul fișierului prelucrat """
    return rand_ltx.endswith('.baby')

# În sursa StarDict rândurile albe sunt, de fapt, comenzi.
# Rândul alb cere construirea unei noi intrări în dicționar.
def este_rand_alb(rand_ltx):
    """ Funcţie booleană pentru a stabili dacă rândul este alb """
    if rand_ltx.startswith('\n'):
        return 1
    else:
        return 0

# Funcție pentru a vedea dacă rândul este un comentariu
def este_comentariu(rand_ltx):
    """ Funcţie booleană pentru a stabili dacă rândul este comentariu """
    if rand_ltx.startswith('#'):
        return 1
    else:
        return 0

# Extragem titlul articolului din corpul său
# Intenția este să nu folosim etichetele definițiilor din fișierul sql
def extrage_titlu(abc):
    """ Reținem doar capul definiției; tăiem coada """
    intrare = re.compile('(~|,|\(|</b>|</B>|<sup>|<SUP>).*$')
    abc = intrare.sub('', abc)
    return abc

# Eliminăm marcajul acolo unde nu este necesar (în titlu)
# Și elemente de început superflue
def elimina_marcaj(abc):
    """ Curățăm titlul de resturi de marcaj """
    intrare = re.compile('<.*?>')
    abc = intrare.sub('', abc)
    abc = re.sub('^a se ', '', abc)
    abc = re.sub('^a-i ', '', abc)
    abc = re.sub('^a-și ', '', abc)
    abc = re.sub('^a ', '', abc)
    abc = abc.replace(' /', '|')
    abc = abc.replace('/ ', '|')
    abc = abc.replace(' / ', '|')
    abc = abc.replace('/', '|')
    abc = re.sub('\|[ ]*$', '', abc)
    return abc

# Definim o functie pentru subtituţii pentru a elimina accente etc.
def pune_semne_stardict(semne):
    """ Introduce semne compatibile cu StarDict """
    semne = semne.replace('Á', 'A')
    semne = semne.replace('á', 'a')
    semne = semne.replace('Ắ', 'Ă')
    semne = semne.replace('ắ', 'ă')
    semne = semne.replace('Ấ', 'Â')
    semne = semne.replace('ấ', 'â')
    semne = semne.replace('É', 'E')
    semne = semne.replace('é', 'e')
    semne = semne.replace('Í', 'I')
    semne = semne.replace('í', 'i')
    semne = semne.replace('Î', 'Î')
    semne = semne.replace('î', 'î')
    semne = semne.replace('Ó', 'O')
    semne = semne.replace('ó', 'o')
    semne = semne.replace('Ú', 'U')
    semne = semne.replace('ú', 'u')
    semne = semne.replace('//', '')
    semne = semne.replace('\\', '')
    return semne

# Definim o functie pentru a scrie numai cu majuscule
def numai_majuscule(semne):
    """ Introduce semne compatibile cu StarDict """
    semne = semne.replace('ă', 'Ă')
    semne = semne.replace('â', 'Â')
    semne = semne.replace('î', 'Î')
    semne = semne.replace('ș', 'Ș')
    semne = semne.replace('ț', 'Ț')
    semne = semne.replace('à', 'À')
    semne = semne.replace('á', 'A')
    semne = semne.replace('ö', 'Ö')
    semne = semne.upper()
    return semne

# Modificăm bword pentru a nu rupe trimiterile
def modifica_bword(abc):
    """ Ajustăm conținutul bword """
    """ Nu foarte șmecher (tricky) dar inteligibil"""
    trimitere = re.compile(u'(<a href="bword://.*?">)')    # atn.la greedy
    destinatie = re.compile(u'<a href="bword://(.*?)">')
    lista = trimitere.split(abc)
    abc_nou = ''
    for index in range(len(lista)):
        if re.search('bword', lista[index]):       # este și o precauție aici
            lista[index] = destinatie.sub('\\1', lista[index])
            lista[index] = numai_majuscule(lista[index])
            lista[index] = '<a href="bword://' + lista[index] + '">'
        abc_nou += lista[index]
    return abc_nou

# Modificări în corpul definițiilor
def mai_estetic(semne):                            # modificări după gust
    semne = re.sub('^<b>!', '<b>', semne)
    semne = re.sub('^<b>\*', '<b>', semne)
    semne = re.sub('([^:])//', '\\1', semne)       # atn. ://bword   !!!
    return semne

# Verificăm dacă este OK capul definiției lexicale
def antet_def_ok(antet):
    """ Funcţie booleană pentru rezultatul verificării antetului """
    if antet != '' and antet != 'A':             # TODO ceva mai subtil
        return 1
    else:
        return 0

# Punem în capul titlului vechi capul definiției lexicale
def schimba_cap_titlu(abc,cap):
    """ Pune un cap nou în titluri cu alternative """
    cap = cap.strip()                       # elimină semne care pot încurca
    if abc.find('|') != -1:
        titlu  = re.compile('^.*?\|')       # tipar pentru capul intrării
        if antet_def_ok(cap):
            cap += '|'
            abc = titlu.sub(cap, abc)
        else:
            corp = re.compile('\|.*$')      # tipar pentru corpul intrării
            capul = corp.sub('', abc)       # retezăm corpul; rămâne capul
            capul = capul.strip()           # chelit
            capul += '|'                    # introdu separatorul
            capul = numai_majuscule(capul)
            abc = titlu.sub(capul, abc)
    else:
        if antet_def_ok(cap):
            abc = cap + '\n'
        else:
            abc = numai_majuscule(abc)
    return abc

# S-a terminat definirea de funcţii.

# Definim constante pentru a putea scrie antetul sursei dicționarului
BLANK_LINE = '\n'
BOOK_NAME = '#bookname=dexonline.ro\n'
VERSION = '#version=3.0.0\n'
STRIP_METHOD = '#stripmethod=keep\n'
SAME_TYPE_SEQUENCE = '#sametypesequence=h\n'
DB_AUTHOR = '#author=\n'
SRC_AUTHOR_EMAIL = '#email=\n'
WEBSITE = '#website=\n'
SRC_DESCRIPTION = '#description=Acesta este o adaptare\
 a bazei de date DEX online sub licența GPL.\n'
DATA = '#date=' + time.strftime("%Y-%m-%d") + '\n'

# În urma apelului din linia de comandă la momentul
print time.ctime()
# începe procesul de prelucrare efectivă a fişierului src
if __name__ == "__main__":
    nume_fisier_in = sys.argv[1]         # preluarea numelui fişierului src
    if este_baby(nume_fisier_in):
        fisier = open(nume_fisier_in,'r')# deschiderea fişierului src
    else:
        print 'No baby file!'
        exit()
    text_src = fisier.readlines()        # preluarea conţinutului fişierului
    fisier.close()                       # închiderea fişierului src

# Rezultatul prelucrării va sta în text_bab
    text_bab = []

# Deschidem sursa și o prelucrăm
    len_src = len(text_src)
    if len_src == 0:
        print 'Nu avem ce prelucra!'
        sys.exit(1)                              # ieșim cu un mesaj de eroare

# Verificăm dacă primul rând este alb
    index = 0
    if not este_rand_alb(text_src[index]):
        print 'Primul rând trebuie să fie alb!'
        sys.exit(1)

# Sărim peste comentarii (dacă ele există la locul lor!)
    index = 1
    if este_comentariu(text_src[index]):
        while este_comentariu(text_src[index]) and (index+1 < len_src):
            index += 1
    else:
        index -= 1                              # ne întoarcem pe primul rând

# Verificăm dacă am ajuns la un rând alb
    if not este_rand_alb(text_src[index]):
        print 'După comentarii trebuie pus un rând alb!'
        sys.exit(1)

# Prelucrăm perechile label+definition din src.baby
    while index < len_src:
        if este_rand_alb(text_src[index]):
            if (index+1 < len_src) and este_rand_alb(text_src[index+1]):
                print '%d este alb din greșeală!' % (index+2)# în fișier rând+1
                break
            text_bab += ['\n']
            index += 1
            continue
        elif not este_rand_alb(text_src[index]):
            if (index+1 < len_src) and este_rand_alb(text_src[index+1]):
                print '%d : lipsește definiția!' % (index+2)# în fișier rând+1
                break
            elif (index+2 < len_src) and not este_rand_alb(text_src[index+2]):
                print '%d este plin din greșeală!' % (index+3)# în fișier rând+1
                break
            src_ttl = text_src[index]
            src_art = text_src[index+1]
            src_art = mai_estetic(src_art)
            ttl_art = extrage_titlu(src_art)
            ttl_art = pune_semne_stardict(ttl_art)
            ttl_art = elimina_marcaj(ttl_art)
            ttl_art = numai_majuscule(ttl_art)
            ttl_nou = schimba_cap_titlu(src_ttl, ttl_art)
            text_bab += [ttl_nou]
            if re.search('bword', src_art):
                src_art = modifica_bword(src_art)
            text_bab += [src_art]
            index += 2

# În continuare este pregătit pentru scriere fişierul sursă al dicționarului
    nume_fisier_out = nume_fisier_in.replace('.baby', '.babylon')
    fisier_out = open(nume_fisier_out,'w')

    # Scriem antetul sursei în format babylon a dicționarului StarDict
    fisier_out.write(BLANK_LINE)
    fisier_out.write(BOOK_NAME)
    fisier_out.write(VERSION)
    fisier_out.write(STRIP_METHOD)
    fisier_out.write(SAME_TYPE_SEQUENCE)
#    fisier_out.write(DB_AUTHOR)
#    fisier_out.write(SRC_AUTHOR_EMAIL)
#    fisier_out.write(WEBSITE)
    fisier_out.write(DATA)
    fisier_out.write(SRC_DESCRIPTION)
    # Scriem textul sursei StarDict
    for i in range(len(text_bab)): 
        fisier_out.write(text_bab[i])
    
    # închidem fişierul cu sursa dicționarului
    fisier_out.close()

# Am încheiat prelucrarea fişierului src şi scrierea fişierului corectat
# Gata la momentul
print time.ctime()
# Final
